ebfdr <- function(obs.score,
                  null.score,
                  mass,
                  estimate.p0=FALSE,
                  p0.step=1,
                  p0=1){
  
  obs.score <- obs.score[!is.na(obs.score)]
  null.score <- null.score[!is.na(null.score)]
  xxx <- sort(obs.score)
  nx <- length(xxx)
  obs.tail <- null.tail <- rep(0,nx)
  
  for (i in 1:nx ){
    null.tail[i] <- sum ( null.score >= xxx[i] ) / mass
    obs.tail[i] <- sum ( obs.score >= xxx[i] )
  }

  Fdr <- p0 * null.tail / obs.tail
  
  dens <- density(xxx)
  steps0 <- c(null.tail[1:(nx-1)]-null.tail[2:nx],0)
  if (sum(steps0) == 0 ) {
    fdr <- rep(0,nx)
    }
  else{
    dens0 <- density(xxx,weights=steps0/sum(steps0))
    fdr.grid <- p0 * ( max(null.tail) * ( dens0$y - min(dens0$y) ) ) /
      ( nx * ( dens$y - min(dens$y) + 1.0e-4) )
    ngrid <- length(dens$x)
    fff <- ff0 <- fdr <- rep(0,nx)
    for (i in 1:length(xxx) ){
      iclose <- max ( (1:ngrid) [ abs(dens$x-xxx[i]) == min(abs(dens$x-xxx[i])) ] )
      fff[i] <-  dens$y[iclose]
      ff0[i] <-  dens0$y[iclose]
      fdr[i] <- fdr.grid[iclose]
    }
  }

  # monotonization (to the right of the max Fdr)
  
  maxFdrID <- max( (1:nx)[ Fdr==max(Fdr) ] )
  for (i in (maxFdrID+1):nx ){
    Fdr[i] <- min(Fdr[i],Fdr[i-1])
    fdr[i] <- min(fdr[i],fdr[i-1])
  }
  Fdr[Fdr>1] <- 1
  fdr[fdr>1] <- 1

  # estimate p0

  if (estimate.p0) {
    scores <- c(obs.score, null.score)
    maximum <- ceiling(max(scores))
    minimum <- floor(min(scores))
    
    hh <- hist(obs.score,breaks=seq(minimum,maximum,p0.step),
               plot=FALSE)
    h0 <- hist(null.score,breaks=seq(minimum,maximum,p0.step),
               plot=FALSE)
    
    nonzero <- hh$density > 0

    p0 = 1 / max(h0$density[nonzero]/hh$density[nonzero], na.rm = TRUE)

  }

  out <- as.data.frame(cbind(xxx,obs.tail,null.tail,Fdr,fdr,p0))
  rownames(out) <- names(xxx)
  colnames(out) <- c("Score","F","F0","Fdr","fdr","p0")

  return(out)

}
